/*
 * Programming in Java, Jacobs University, 2015
 *
 * This class demonstrates the use of Locks in Java as a low-level
 * synchronization primitive.
 */

import java.lang.Thread;
import java.lang.Runnable;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;

public class D implements Runnable
{
    static private String buffer = "";
    static private final Lock lock = new ReentrantLock();
    private String s = null;
    private int ms = 0;
    
    public D(String s)
    {
	this(s, 1000);
    }

    public D(String s, int ms)
    {
	this.s = s;
	this.ms = ms;
    }

    public void run()
    {
	int len = s.length();

	for (int i = 0; i < len && ! Thread.currentThread().isInterrupted(); i++) {
	    try {
		if (lock.tryLock(10, TimeUnit.SECONDS)) {
		    buffer += s + " ";
		    lock.unlock();
		} else {
		    System.err.println("skipping - failed to acquire lock");
		}
	    } catch (InterruptedException e) {
		Thread.currentThread().interrupt();
	    }
	    try {
		TimeUnit.MILLISECONDS.sleep(ms);
	    } catch(InterruptedException e) {
		Thread.currentThread().interrupt();
	    }
	}
    }

    public static void main(String[] args)
	throws Exception
    {
	ExecutorService pool = Executors.newFixedThreadPool(5);
	for (String arg: args) {
	    pool.execute(new D(arg));
	}

        pool.shutdown();
	while (! pool.awaitTermination(4242, TimeUnit.MILLISECONDS)) {
	    System.err.println("timeout while waiting for termination: retrying...");
        }
	System.out.println(buffer);
    }
}
